@@ -4,7 +4,6 @@ import copy
 import math
 from tqdm import tqdm
 import pyvista as pv
-import time
 
 class scapula():
     def __init__(self, file_name):
@@ -466,6 +465,7 @@ class scapula():
         o3d.io.write_triangle_mesh('guide.stl', self.guide_mesh)
         o3d.io.write_triangle_mesh('handle.stl', self.cylinder2)
         o3d.io.write_triangle_mesh('mesh.stl', self.mesh)
+        o3d.io.write_triangle_mesh('jizuo.stl', self.jizuo)
 
     def go_back(self):
         n = len(self.change)
@@ -476,8 +476,120 @@ class scapula():
                 self.cylinder = copy.deepcopy(self.cylinder).translate(p[1])
                 self.cylinder2 = copy.deepcopy(self.cylinder2).translate(p[1])
                 self.guide_mesh = copy.deepcopy(self.guide_mesh).translate(p[1])
+                self.jizuo = copy.deepcopy(self.jizuo).translate(p[1])
             else:
                 self.mesh = self.mesh.rotate(p[1], center=(0, 0, 0))
                 self.cylinder = self.cylinder.rotate(p[1], center=(0, 0, 0))
                 self.cylinder2 = self.cylinder2.rotate(p[1], center=(0, 0, 0))
-                self.guide_mesh = self.guide_mesh.rotate(p[1], center=(0, 0, 0))
\ No newline at end of file
+                self.guide_mesh = self.guide_mesh.rotate(p[1], center=(0, 0, 0))
+                self.jizuo = self.jizuo.rotate(p[1], center=(0, 0, 0))
+    
+    def find_jizuo(self, filename):
+        def dis(x, y):
+            return np.sqrt((x[0] - y[0]) ** 2 + (x[1] - y[1]) ** 2 + (x[2] - y[2]) ** 2)
+
+        def find_angle(p1, p2, p3):
+            l1 = dis(p1, p2); l2 = dis(p2, p3); l3 = dis(p1, p3)
+            cos = (l1 ** 2 + l2 ** 2 - l3 ** 2) / (2 * l1 * l2)
+            return math.acos(cos)/np.pi
+
+        # 改变基座位置
+        def change_jizuo(mesh_jizuo1):
+            a = - mesh_jizuo1.get_center() + [0, 0, 0]
+            mesh_jizuo = copy.deepcopy(mesh_jizuo1).translate(tuple(a))
+            mesh_jizuo.compute_vertex_normals()
+            return mesh_jizuo
+
+        # 旋转基座
+        def rotate_jizuo(mesh_jizuo, direction_vector):
+            point_coordinate = [0, 0, 0]
+            # 向量OC为基座方向向量
+            vector_oc = mesh_jizuo.get_center() - bottom_center
+            vector_oc = [vector_oc[0] / vector_oc[2], vector_oc[1] / vector_oc[2], 1]
+            print(vector_oc)
+
+            # 向量OD为钉子方向向量
+            vector_od = [direction_vector[0] / direction_vector[2], direction_vector[1] / direction_vector[2], 1]
+            print(vector_od)
+
+            # 基座方向向量与z轴夹角
+            theta = find_angle(vector_oc, [0, 0, 0], [0, 0, 1])
+
+            # 钉子方向向量与z轴夹角
+            beta = find_angle(vector_od, [0, 0, 0], [0, 0, 1])
+
+            # 第一次旋转
+            vector_oc2 = [0, np.sin(np.pi * theta) / np.cos(np.pi * theta), 1]
+            alpha = find_angle(vector_oc, [0, 0, 1], vector_oc2)
+            
+            R = self.mesh.get_rotation_matrix_from_xyz((0, 0, - np.pi * alpha))
+            mesh_jizuofirst = copy.deepcopy(mesh_jizuo)
+            mesh_jizuofirst.rotate(R, center=point_coordinate)
+
+            # 第二次旋转
+            R = self.mesh.get_rotation_matrix_from_xyz((np.pi * theta - np.pi * beta, 0, 0))
+            mesh_jizuosecond = copy.deepcopy(mesh_jizuofirst)
+            mesh_jizuosecond.rotate(R, center=point_coordinate)
+
+            # 第三次旋转
+            vector_oc3 = [0, np.sin(np.pi * beta) / np.cos(np.pi * beta), 1]
+            delte = find_angle(vector_oc3, [0, 0, 1], vector_od)
+            if vector_od[0] < 0:
+                delte = - delte
+
+            R = self.mesh.get_rotation_matrix_from_xyz((0, 0, - np.pi * delte))
+            mesh_jizuothird = copy.deepcopy(mesh_jizuosecond)
+            mesh_jizuothird.rotate(R, center=point_coordinate)
+            return mesh_jizuothird
+        
+        def find_center(p1, p2, p3):
+            x1 = p1[0];y1 = p1[1];z1 = p1[2]
+            x2 = p2[0];y2 = p2[1];z2 = p2[2]
+            x3 = p3[0];y3 = p3[1];z3 = p3[2]
+            a1 = (y1*z2 - y2*z1 - y1*z3 + y3*z1 + y2*z3 - y3*z2)
+            b1 = -(x1*z2 - x2*z1 - x1*z3 + x3*z1 + x2*z3 - x3*z2)
+            c1 = (x1*y2 - x2*y1 - x1*y3 + x3*y1 + x2*y3 - x3*y2)
+            d1 = -(x1*y2*z3 - x1*y3*z2 - x2*y1*z3 + x2*y3*z1 + x3*y1*z2 - x3*y2*z1)
+            a2 = 2 * (x2 - x1)
+            b2 = 2 * (y2 - y1)
+            c2 = 2 * (z2 - z1)
+            d2 = x1*x1 + y1*y1 + z1*z1 - x2*x2 - y2*y2 - z2*z2
+            a3 = 2 * (x3 - x1)
+            b3 = 2 * (y3 - y1)
+            c3 = 2 * (z3 - z1)
+            d3 = x1*x1 + y1*y1 + z1*z1 - x3*x3 - y3*y3 - z3*z3
+            x = -(b1*c2*d3 - b1*c3*d2 - b2*c1*d3 + b2*c3*d1 + b3*c1*d2 - b3*c2*d1) / (a1*b2*c3 - a1*b3*c2 - a2*b1*c3 + a2*b3*c1 + a3*b1*c2 - a3*b2*c1)
+            y = (a1*c2*d3 - a1*c3*d2 - a2*c1*d3 + a2*c3*d1 + a3*c1*d2 - a3*c2*d1) / (a1*b2*c3 - a1*b3*c2 - a2*b1*c3 + a2*b3*c1 + a3*b1*c2 - a3*b2*c1)
+            z = -(a1*b2*d3 - a1*b3*d2 - a2*b1*d3 + a2*b3*d1 + a3*b1*d2 - a3*b2*d1) / (a1*b2*c3 - a1*b3*c2 - a2*b1*c3 + a2*b3*c1 + a3*b1*c2 - a3*b2*c1)
+            return x, y, z
+
+        
+        #读入基座文件
+        mesh_jizuo1 = o3d.io.read_triangle_mesh(filename)
+        mesh_jizuo1.compute_vertex_normals()
+
+        #移动基座到球心
+        mesh_jizuo = change_jizuo(mesh_jizuo1)
+
+        # o3d.visualization.draw_geometries([mesh_cylindernew, mesh, mesh_sphere2, mesh_sphere,mesh_jizuo])
+
+        #计算基座定向边框的顶点
+        obb = mesh_jizuo.get_oriented_bounding_box()
+        m1 = np.asarray(obb.get_box_points())[0]
+        m2 = np.asarray(obb.get_box_points())[1]
+        m3 = np.asarray(obb.get_box_points())[2]
+
+        #计算基座底面中心
+        x1, y1, z1 = find_center(m1, m2, m3)
+        bottom_center = [x1, y1, z1]
+
+        #计算钉子方向向量
+        direction_vector = np.asarray(self.cylinder.vertices)[0] - np.asarray(self.cylinder.vertices)[1]
+        print(direction_vector)
+
+        #旋转基座
+        mesh_jizuonew = rotate_jizuo(mesh_jizuo, direction_vector)
+        self.jizuo = copy.deepcopy(mesh_jizuonew)
+
+        #展示一下
+        # o3d.visualization.draw_geometries([mesh_cylindernew, mesh, mesh_sphere2, mesh_sphere, mesh_jizuonew])
\ No newline at end of file
